@@ -5,17 +5,14 @@ use warnings;
 
 use Exporter;
 our @ISA = qw(Exporter);
-our @EXPORT = qw(bgs_call bgs_back bgs_wait bgs_break);
+our @EXPORT = qw(bgs_call bgs_back bgs_wait);
 
-our $VERSION = '0.07';
+our $VERSION = '0.01';
 
 use IO::Select;
 use Storable qw(freeze thaw);
-use POSIX ":sys_wait_h";
 
 
-$SIG{CHLD} = "IGNORE";
-
 my $sel = IO::Select->new();
 my %callbacks = (); 
 
@@ -23,28 +20,26 @@ my %callbacks = ();
 sub bgs_call(&$) {
 	my ($sub, $callback) = @_;
 
-	pipe my $from_kid_fh, my $to_parent_fh or die "pipe: $!";
-
-	my $kid_pid = fork;
+	my $kid_pid = open my $fh, "-|";
 	defined $kid_pid or die "Can't fork: $!";
 
 	if ($kid_pid) {
-		$sel->add($from_kid_fh);
-		$callbacks{$from_kid_fh} = $callback;
+		
+		$sel->add($fh);
+		$callbacks{$fh} = $callback;
 	} else {
-		binmode $to_parent_fh;
-		print $to_parent_fh freeze \ scalar $sub->();
-		close $to_parent_fh;
+		
+		binmode STDOUT;
+		print STDOUT freeze \ $sub->();
+		close STDOUT;
 		exit;
 	}
-	return $kid_pid;
 }
 
 sub bgs_back(&) { shift }
 
 
 sub bgs_wait() {
-	local $SIG{PIPE} = "IGNORE";
 	my %from_kid;       
 	my $buf;            
 	my $blksize = 1024; 
@@ -55,18 +50,14 @@ sub bgs_wait() {
  				push @{$from_kid{$fh}}, $buf;
  			} elsif (defined $len) { 
 				$sel->remove($fh); 
-				close $fh or warn "Kid is existed: $?";
+				                   
+				close $fh or die "Kid exited: $?"; 
 
-				if (exists $from_kid{$fh}) {
-	 				my $r = join "", @{$from_kid{$fh}};
- 					delete $from_kid{$fh};
-					$callbacks{$fh}->(${thaw $r});
-				} else {
-					$callbacks{$fh}->();
-				}
+ 				my $r = join "", @{$from_kid{$fh}};
+ 				delete $from_kid{$fh};
 
+ 				$callbacks{$fh}->(${thaw $r});
  				delete $callbacks{$fh};
-
  			} else {
  				die "Can't read '$fh': $!";
  			}
@@ -75,13 +66,6 @@ sub bgs_wait() {
 }
 
 
-sub bgs_break() {
-	local $SIG{TERM} = "IGNORE";
-	kill 15, -$$;
-	1 while waitpid(-1, WNOHANG) > 0;
-}
-
-
 1;
 
 
@@ -128,9 +112,8 @@ B<bgs_call>, and it executes within this child process.
 The subroutine must return either a B<scalar> or a B<reference>!
 
 The answer of the subroutine passes to the callback subroutine as an argument.
-If a child process ended without bgs_call value returning, than bgs_back subprogram is called without argument.
 
-bgs_call return PID of child proces.
+B<Attention>! The subroutine must print nothing in STDOUT!
 
 =head2 bgs_back
 
@@ -144,17 +127,13 @@ as an argument.
 Call of bgs_wait() reduces to child processes answers wait and
 callback subroutines execution.
 
-=head2 bgs_break
-
-kill all child processes.
-
 =head1 AUTHOR
 
 Nick Kostirya
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011 by Nick Kostirya
+Copyright (C) 2008 by Nick Kostirya
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
